import type { SyntaxHighlighterProps } from 'src/types/shiki'
import React from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'
import { useSyntaxHighlighter } from '../../hooks/useSyntaxHighlighter'
import { convertShikiTheme } from '../../utils/style-transformer'
import SyntaxLine from './SyntaxLine'

const monospaceFont = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

const styles = StyleSheet.create({
  container: {
    flex: 1,
    minHeight: 20,
  },
  scrollContent: {
    flexGrow: 1,
    padding: 8,
  },
  contentContainer: {
    minWidth: '100%',
  },
  errorText: {
    padding: 8,
    color: '#ff6b6b',
  },
  loadingText: {
    padding: 8,
    color: '#666666',
  },
})

function SyntaxHighlighter({ text, language, languageId, theme, fontSize = 14 }: SyntaxHighlighterProps) {
  // Convert theme on mount
  const themeStyles = React.useMemo(() => convertShikiTheme(theme ?? {}), [theme])
  const { tokens, error, isLoading } = useSyntaxHighlighter({
    text,
    language,
    languageId,
    theme,
  })

  if (error) {
    return (
      <Text
        style={[
          styles.errorText,
          {
            fontFamily: monospaceFont,
            fontSize,
          },
        ]}
      >
        Error:
        {' '}
        {error}
      </Text>
    )
  }

  if (isLoading) {
    return (
      <Text
        style={[
          styles.loadingText,
          {
            fontFamily: monospaceFont,
            fontSize,
          },
        ]}
      >
        Loading...
      </Text>
    )
  }

  return (
    <View style={[styles.container, { backgroundColor: themeStyles.backgroundColor }]}>
      <ScrollView horizontal showsHorizontalScrollIndicator={Platform.OS !== 'web'} contentContainerStyle={{ flexGrow: 1 }}>
        <View style={styles.contentContainer}>
          {tokens.map((line, lineIndex) => (
            <SyntaxLine key={`line-${lineIndex}`} line={line} fontSize={fontSize} isLastLine={lineIndex === tokens.length - 1} themeStyles={themeStyles} />
          ))}
        </View>
      </ScrollView>
    </View>
  )
}

export default SyntaxHighlighter
