import type { SyntaxHighlighterProps, ThemedToken, TokensResult } from '../types/shiki'
import { useEffect, useState } from 'react'
import { createHighlighter } from '../syntax'

export function useSyntaxHighlighter({ text, language, languageId, theme }: SyntaxHighlighterProps) {
  const [tokens, setTokens] = useState<ThemedToken[][]>([])
  const [error, setError] = useState<string>()
  const [isLoading, setIsLoading] = useState(true)

  useEffect(() => {
    let mounted = true
    setIsLoading(true)

    async function highlight() {
      if (!text) {
        setIsLoading(false)
        setTokens([])
        return
      }

      try {
        const highlighter = await createHighlighter({
          langs: [language],
          themes: theme ? [theme] : [],
        })

        const result = highlighter.codeToTokens(text, {
          lang: languageId,
          theme: theme || 'none',
        }) as TokensResult

        if (mounted && result.tokens) {
          setTokens(result.tokens)
          setError(undefined)
        }
      }
      catch (err) {
        console.error('Highlighting error:', err)
        if (mounted) {
          setError(err instanceof Error ? err.message : 'Syntax highlighting failed')
          setTokens([])
        }
      }
      finally {
        if (mounted) {
          setIsLoading(false)
        }
      }
    }

    highlight()
    return () => {
      mounted = false
    }
  }, [text, language, languageId, theme])

  return { tokens, error, isLoading }
}
