import type { TextStyle } from 'react-native'
import type { ThemeRegistrationAny } from 'shiki'

export interface RNTokenStyle {
  color?: string
  backgroundColor?: string
  fontWeight?: TextStyle['fontWeight']
  fontStyle?: TextStyle['fontStyle']
}

const ALLOWED_STYLE_PROPERTIES = {
  'color': true,
  'background': true,
  'background-color': true,
  'font-weight': true,
  'font-style': true,
} as const

function transformValue(key: string, value: string): any {
  switch (key) {
    case 'background':
    case 'background-color':
      return ['backgroundColor', value]
    case 'font-weight':
      return ['fontWeight', value === 'bold' ? 'bold' : 'normal']
    case 'font-style':
      return ['fontStyle', value === 'italic' ? 'italic' : 'normal']
    default:
      return [key, value]
  }
}

export function convertShikiTheme(theme: ThemeRegistrationAny): {
  backgroundColor: string
  defaultColor: string
  styles: Record<string, RNTokenStyle>
} {
  // Extract theme colors
  const backgroundColor = theme.bg || '#1e1e1e'
  const defaultColor = theme.fg || '#d4d4d4'

  // Convert token colors
  const styles: Record<string, RNTokenStyle> = {}

  theme.settings?.forEach((setting) => {
    if (!setting.scope || !setting.settings)
      return

    const scopes = Array.isArray(setting.scope) ? setting.scope : [setting.scope]
    const style: RNTokenStyle = {}

    // Convert CSS-style properties to RN style properties
    Object.entries(setting.settings).forEach(([key, value]) => {
      if (ALLOWED_STYLE_PROPERTIES[key as keyof typeof ALLOWED_STYLE_PROPERTIES]) {
        const [rnKey, rnValue] = transformValue(key, value as string)
        style[rnKey as keyof RNTokenStyle] = rnValue
      }
    })

    // Apply style to each scope
    scopes.forEach((scope) => {
      styles[scope] = style
    })
  })

  return {
    backgroundColor,
    defaultColor,
    styles,
  }
}
